@@ -6,9 +6,11 @@ import errno
 import subprocess
 import re
 from argparse import ArgumentParser
+from zipfile import ZipFile
 
 from datetime import datetime
 from termcolor import colored, cprint
+from six import BytesIO
 from six.moves.urllib.parse import quote
 from six.moves.urllib.request import urlopen
 from six.moves.urllib.error import HTTPError
@@ -16,9 +18,10 @@ from six.moves import map
 # Required for Windows
 import colorama
 
-DEFAULT_REMOTE = "https://raw.github.com/tldr-pages/tldr/master/pages"
+DEFAULT_REMOTE = "https://raw.githubusercontent.com/tldr-pages/tldr/master/pages"
 USE_CACHE = int(os.environ.get('TLDR_CACHE_ENABLED', '1')) > 0
 MAX_CACHE_AGE = int(os.environ.get('TLDR_CACHE_MAX_AGE', 24))
+DOWNLOAD_CACHE_LOCATION = 'https://tldr-pages.github.io/assets/tldr.zip'
 
 COMMAND_FILE_REGEX = re.compile(r'(?P<command>^.+?)_(?P<platform>.+?)\.md$')
 
@@ -65,7 +68,8 @@ rows, columns = get_terminal_size()
 os_directories = {
     "linux": "linux",
     "darwin": "osx",
-    "sunos": "sunos"
+    "sunos": "sunos",
+    "win32": "windows",
 }
 
 
@@ -163,7 +167,7 @@ def get_platform():
 
 def get_page(command, remote=None, platform=None):
     if platform is None:
-        platform = ["common", get_platform()]
+        platform = [get_platform(), "common"]
     for _platform in platform:
         if _platform is None:
             continue
@@ -259,23 +263,46 @@ def update_cache(remote=None):
             download_and_store_page_for_platform(command, platform, remote=remote)
             print('Updated cache for %s (%s) from %s' % (command, platform, remote))
         except Exception:
-            print('Error: Unable to get %s (%s) from %s' % (command, platform, remote))
+            sys.exit('Error: Unable to get %s (%s) from %s' % (command, platform, remote))
 
 
+def download_cache():
+    cache_path = get_cache_dir()
+    if not os.path.exists(cache_path):
+        return
+    try:
+        req = urlopen(DOWNLOAD_CACHE_LOCATION)
+        zipfile = ZipFile(BytesIO(req.read()))
+        pattern = re.compile(r'pages/(.+)/(.+)\.md')
+        cached = 0
+        for entry in zipfile.namelist():
+            match = pattern.match(entry)
+            if match:
+                store_page_to_cache(zipfile.read(entry), match.group(2), match.group(1))
+                cached += 1
+        print("Updated cache for {:d} entries".format(cached))
+    except Exception:
+        sys.exit("Error: Unable to update cache from tldr site")
+
 
 def main():
-    parser = ArgumentParser(description="Python command line client for tldr")
+    parser = ArgumentParser(prog="tldr", description="Python command line client for tldr")
 
     parser.add_argument('-u', '--update_cache',
                         action='store_true',
                         help="Update the cached commands")
 
-    parser.add_argument('-o', '--os',
+    parser.add_argument('--download_cache',
+                        action='store_true',
+                        help='Downloads and caches all tldr pages from tldr site')
+
+    parser.add_argument('-p', '--platform',
                         nargs=1,
                         default=None,
                         type=str,
-                        choices=['linux', 'osx', 'sunos'],
-                        help="Override the operating system [linux, osx, sunos]")
+                        choices=['linux', 'osx', 'sunos', 'windows', 'common'],
+                        metavar='PLATFORM',
+                        help="Override the operating system [linux, osx, sunos, windows, common]")
 
     parser.add_argument('-s', '--source',
                         default=DEFAULT_REMOTE,
@@ -288,10 +315,20 @@ def main():
                         const=False,
                         help="Override color stripping")
 
+    parser.add_argument('-r', '--render',
+                        default=False,
+                        action='store_true',
+                        help='Render local markdown files'
+                        )
+
     options, rest = parser.parse_known_args()
 
     colorama.init(strip=options.color)
 
+    if options.download_cache:
+        download_cache()
+        return
+
     if options.update_cache:
         update_cache(remote=options.source)
         return
@@ -299,25 +336,36 @@ def main():
     parser.add_argument(
         'command', type=str, nargs='+', help="command to lookup")
 
+    if len(sys.argv) == 1:
+        parser.print_help(sys.stderr)
+        sys.exit(1)
+
     rest = parser.parse_args(rest)
 
-    try:
-        result = get_page('-'.join(rest.command), platform=options.os, remote=options.source)
-        if not result:
-            for command in rest.command:
-                result = get_page(command, platform=options.os, remote=options.source)
-                if not result:
-                    print((
-                        "`{cmd}` documentation is not available. "
-                        "Consider contributing Pull Request to https://github.com/tldr-pages/tldr"
-                    ).format(cmd=command))
-                else:
-                    output(result)
-        else:
-            output(result)
-    except Exception:
-        print("No internet connection detected. Please reconnect and try again.")
-
+    if options.render:
+        for command in rest.command:
+            if os.path.exists(command):
+                with open(command, encoding='utf-8') as open_file:
+                    output(open_file.read().encode('utf-8').splitlines())
+    else:
+        try:
+            result = get_page('-'.join(rest.command), platform=options.platform, remote=options.source)
+            if not result:
+                errors_found = False
+                for command in rest.command:
+                    result = get_page(command, platform=options.platform, remote=options.source)
+                    if not result:
+                        errors_found = True
+                        print((
+                            "`{cmd}` documentation is not available. "
+                            "Consider contributing Pull Request to https://github.com/tldr-pages/tldr"
+                        ).format(cmd=command), file=sys.stderr)
+                    else:
+                        output(result)
+            else:
+                output(result)
+        except Exception:
+            sys.exit("No internet connection detected. Please reconnect and try again.")
 
 if __name__ == "__main__":
     main()
